#!/usr/bin/perl
use warnings;
use strict;
use Getopt::Long;
Getopt::Long::config qw(no_ignore_case);

#############################################################
# define where nagios files are stored and other such goodness
my $nagiosRoot = "/opt/nagios";
my $configFileName = "check_jmx";
#if (@ARGV > 0) {
#        $configFileName = shift (@ARGV);
#}
my $seek_dir = "$nagiosRoot/etc/logchecks";
if (! -d $seek_dir) {
        mkdir $seek_dir, 0755;
}
my ( $logFile , $debug ) ;


# get command line options

{ my ( $needHelp );
  GetOptions(   'configFileName=s'      => \$configFileName,
                'help|h|H|?|HELP'       => \$needHelp,
                'debug'		        => \$debug
        );
  die usage() if $needHelp;
}

my $configFile = "$nagiosRoot/etc/$configFileName.cfg";

print STDERR "debug: configFile = $configFile \n" if $debug ;

sub usage {
  return <<USAGETEXT;
usage: $0 
if no options are given the defaults from this script will be used
the following options are  availble:
        [ -nagiosRoot path ] the path to Nagios Install
        [ -configFileName name ] name of the config file (without the .cfg)
        [ -configFile path/name ] full path and name of the config file (eg /root/config.cfg)
        [ -seek_dir ]  directory to store temp deta
        [ -logFile ]   full path and name of the data file  
        [ -help] this help text
        [ -debug] deisplay debug output
USAGETEXT
}



my %statii = (
        'OK' => 0,
        'WARNING' => 1,
        'CRITICAL' => 2,
        'UNKNOWN' => 3
);
my @statii = ('CRITICAL', 'WARNING', 'UNKNOWN', 'OK');
my @statiiIndex = ('OK','WARNING','CRITICAL','UNKNOWN');
my %logDataValues;

open (CFG, "< $configFile") || die "Failed to open configuration file $configFile: $!";
my @config = (<CFG>);
close (CFG);



my %levelLists;

#sub check_MaxMin;

my $status = $statii{'OK'};
my @output;

#####################################################
#main config file loop (this does most of the "Work")
#ie loopiing through the conf file and running the right check for each metric
# then creating output.
#####################################################
foreach my $entry (@config) {
	chomp ($entry);
	my (%result, $metricName, $column);
        if ($entry =~ /^#/) {
                next;
        } 
#start Max Min check
	elsif ($entry =~ /^MaxMin:/) { 
	 if ($entry  =~ /^(\S+):(\S+):(\d+\.?\d*):(\d+\.?\d*):(\d+\.?\d*):(\d+\.?\d*):$/) {
		my ($metricName,$warnMin,$warnMax,$critMin,$critMax) = ($2,$3,$4,$5,$6);
                
                %result = check_MaxMin ($metricName, $warnMin, $warnMax, $critMin, $critMax);
                
                if (!defined ($levelLists{$result{'statusName'}})) {
                        $levelLists{$result{'statusName'}} = [];
                }
                push (@{$levelLists{$result{'statusName'}}}, "$result{metricName}($result{resultText})");
                if ($result{'status'} > $status) {
                        $status = $result{'status'};
                }
	 } else {die " conf files is incorectly formated ($entry) "; }
        } 
#start Compair two tags by % check
        elsif ($entry =~ /^CompPct:/) { 
         if ($entry  =~ /^(\S+):(\S+):(\S+):(\d+\.?\d*):(\d+\.?\d*):$/) {
                my ($metricName1,$metricName2,$warnPct,$critPct) = ($2,$3,$4,$5);
                
                %result = check_Compair_percent ($metricName1, $metricName2, $warnPct, $critPct);
                
                if (!defined ($levelLists{$result{'statusName'}})) {
                        $levelLists{$result{'statusName'}} = [];
                }
                push (@{$levelLists{$result{'statusName'}}}, "$result{metricName}($result{resultText})");
                if ($result{'status'} > $status) {
                        $status = $result{'status'};
                }
         } else {die " conf files is incorectly formated ($entry) "; }
        } 

#start Inc Test
	elsif ($entry =~ /^Inc:/) { 
	 if ($entry  =~ /^(\S+):(\S+):(\d+):$/) {
		my ($metricName,$column) = ($2,$3);
		%result = check_Inc ($column, $metricName);

                if (!defined ($levelLists{$result{'statusName'}})) {
                        $levelLists{$result{'statusName'}} = [];
                }
                push (@{$levelLists{$result{'statusName'}}}, "$result{metricName}($result{resultText})");
                if ($result{'status'} > $status) {
                        $status = $result{'status'};
	}

	 } else {die " conf files is incorectly formated ($entry) "; }

        } 
#start Ok Fail
	elsif ($entry =~ /^OkFail:/) { 
	 if ($entry  =~ /^(\S+):(\S+):$/) {
		my ($metricName) = ($2);
		%result = check_OkFail ($metricName);

                if (!defined ($levelLists{$result{'statusName'}})) {
                        $levelLists{$result{'statusName'}} = [];
                }
                push (@{$levelLists{$result{'statusName'}}}, "$result{metricName}($result{resultText})");
                if ($result{'status'} > $status) {
                        $status = $result{'status'};
	}

	 } else {die " conf files is incorectly formated ($entry) "; }

        } 
#start random string
	elsif ($entry =~ /^string:/) { 
	 if ($entry  =~ /^(\S+):(\S+):(\S+):(\S+):(\S+):$/) {
		my ($metricName, $okString, $warnStreing, $critString) = ($2,$3,$4,$5);
		%result = check_string ($metricName, $okString, $warnStreing, $critString);

                if (!defined ($levelLists{$result{'statusName'}})) {
                        $levelLists{$result{'statusName'}} = [];
                }
                push (@{$levelLists{$result{'statusName'}}}, "$result{metricName}($result{resultText})");
                if ($result{'status'} > $status) {
                        $status = $result{'status'};
	}

	 } else {die " conf files is incorectly formated ($entry) "; }

        } 
#importdata
	elsif ($entry =~ /^jmx_logFile\s*=\s*([^#]+)/) {
                $logFile= $1;
		chomp ($logFile);
#############################################
#a sneeky import of the data from the log file 
#i should probbly add a check and such for this (dont like braking out but its the cleanist way i can see)
# infact maybe a sub to get the latest data and varify thats its current

print STDERR "debug: logFile = $logFile \n" if $debug ;

my $logData = `/usr/bin/tail -1 $logFile`;
chomp ($logData);
my @logDataPairs = split(/ /,$logData);

print STDERR "debug:  @logDataPairs \n" if $debug ;

#
#
#
#my $dateCheck = shift(@logDataPairs);
# need to add a check in here for date and OK string
#
#
my ($tagName, $tagData);

foreach my $dataPair (@logDataPairs) {
  if ($dataPair =~ /^\?$/ ) {
     next; }
  ($tagName,$tagData) = split(/:/,$dataPair,2);
  $logDataValues{$tagName} = $tagData;

 # test to dump tag names / values 
 print STDERR "debug: data pair :\t  \"$tagName\"  =  \"$tagData\" \n" if $debug;
}
#############################################

        }
	 else {die "check type not found or conf files is incorectly formated ($entry), "; }

}




my @allOutput;
foreach my $status (@statii) {
        if (defined ($levelLists{$status})) {
                push (@allOutput, "$status: @{$levelLists{$status}}");
        }
}
print join (", ", @allOutput), "\n";
#print "# TEST DATA = $logData \n" ;
exit ($status);





#####################################
#Sub to check on Max Min types
#####################################

sub check_MaxMin {
        my ($metricName, $warnMin, $warnMax, $critMin, $critMax) = (@_);
	my $metricStatus = 0;
	my ($metricData, $output);

print STDERR "debug: max test data. $metricName, $warnMin, $warnMax, $critMin, $critMax \n" if $debug;

	if ( !defined $logDataValues{$metricName} ) {
	  $metricData = "Tag not found"; $metricStatus = 2 ; $output = "$metricData"; } 
	else {

	$metricData = $logDataValues{$metricName};

	if ( $metricData > $critMax ) { $metricStatus = 2 ; $output = "$metricData is over $critMax"}
	elsif ( $metricData < $critMin ) { $metricStatus = 2 ; $output = "$metricData is under $critMin"}
	elsif ( $metricData > $warnMax ) { $metricStatus = 1 ; $output = "$metricData is over $warnMax"}
	elsif ( $metricData < $warnMin ) { $metricStatus = 1 ; $output = "$metricData is under $warnMin"}
	else {  $output = "$metricData" }
}

        my $statusName = $statiiIndex[$metricStatus];
        my %result = ('status' => $metricStatus, 'statusName' => $statusName, 'metricName' => $metricName, 'resultText' => $output );
        return (%result);
}

#####################################
#Sub to check on Percent types
#####################################

sub check_Compair_percent {
        my ($metricName1, $metricName2, $warnPct, $critPct) = (@_);
        my $metricStatus = 0;
        my ($metricDataPercent, $metricData1, $metricData2,  $output);
print STDERR "debug:  percent test data. $metricName1, $metricName2, $warnPct, $critPct \n" if $debug;

        if ( !defined $logDataValues{$metricName1} ) {
          $metricData1 = "Tag not found"; $metricStatus = 2 ; $output = "$metricData1"; }
        else {
	if ( !defined $logDataValues{$metricName2} ) {
          $metricData2 = "Tag not found"; $metricStatus = 2 ; $output = "$metricData2"; }
	  
        else {

        $metricData1 = $logDataValues{$metricName1};
        $metricData2 = $logDataValues{$metricName2};
		$metricDataPercent = ($metricData2 / $metricData1) * 100 ;
		
	
        if ( $metricDataPercent > $critPct ) { $metricStatus = 2 ; $output = "$metricDataPercent  is over $critPct"}
        elsif ( $metricDataPercent > $warnPct ) { $metricStatus = 1 ; $output = "$metricDataPercent  is over $warnPct"}
#        else {  $output = "$metricDataPercent" }
        else {  $output = int($metricDataPercent +.5)."%" }
} }

        my $statusName = $statiiIndex[$metricStatus];
        my %result = ('status' => $metricStatus, 'statusName' => $statusName, 'metricName' => $metricName2, 'resultText' => $output );
        return (%result);
}

#####################################
#Sub to check on Ok Fail
#####################################

sub check_OkFail {
        my ($metricName) = (@_);
	my  $metricStatus = 2;
	my ($metricData, $output);

	if ( !defined $logDataValues{$metricName} ) {
	  $metricData = "Tag not found"; $metricStatus = 2 ; $output = "$metricData"; } 
	else {
	$metricData = $logDataValues{$metricName};

        if ( $metricData eq "OK" ) { $metricStatus = 0 ; $output = "$metricData" }
        elsif ( $metricData eq "FAIL" ) { $metricStatus = 2 ; $output = "$metricData Has FAILED"}

}

        my $statusName = $statiiIndex[$metricStatus];
        my %result = ('status' => $metricStatus, 'statusName' => $statusName, 'metricName' => $metricName, 'resultText' => $output );
        return (%result);
}

#####################################
#Sub to check on random strings
#####################################

sub check_string {
	my ($metricName, $okString, $warnString, $critString) = (@_);
	my  $metricStatus = 2;
	my ($metricData, $output);

	if ( !defined $logDataValues{$metricName} ) {
	  $metricData = "Tag not found"; $metricStatus = 2 ; $output = "$metricData"; } 
	else {
	  $metricData = $logDataValues{$metricName};

          if ( $metricData eq $critString ) { $metricStatus = 2 ; $output = "$critString"}
          elsif ( $metricData eq $warnString ) { $metricStatus = 1 ; $output = "$warnString"}
          elsif ( $metricData eq $okString ) { $metricStatus = 0 ; $output = "$metricData" }
	  else { $metricStatus = 2 ; $output = "string not known($metricData)"}
	}

        my $statusName = $statiiIndex[$metricStatus];
        my %result = ('status' => $metricStatus, 'statusName' => $statusName, 'metricName' => $metricName, 'resultText' => $output );
        return (%result);
}


#####################################
#Sub to check on Max Min types      
#####################################
                 
sub check_Inc {  
        my ($column, $metricName) = (@_);
        my $metricStatus = 0;
	my (@oldData, $metricData,  $output);

        if ( !defined $logDataValues{$metricName} ) {
          $metricData = "Tag not found"; $metricStatus = 2 ; $output = "$metricData"; }
        else {

        $metricData = $logDataValues{$metricName};
	my $seek_file = "$seek_dir/check_jmx.$configFileName-$metricName.seek" ;

	if (-f $seek_file) {
                open (SEEK,"$seek_file") || die "Failed to open seek file $seek_file: $!";
		@oldData = (<SEEK>);
                close (SEEK);
	} else {
		@oldData = 0 ;
	}
                open (SEEK,">$seek_file") || die "Failed to open for write seek file $seek_file: $!";
                print SEEK "$metricData";
                close (SEEK);
        if ( $metricData <= $oldData[0] ) { $metricStatus = 2 ; $output = "$metricData has not incremented"}
	else { $output = "$metricData"}
}
        my $statusName = $statiiIndex[$metricStatus];
        my %result = ('status' => $metricStatus, 'statusName' => $statusName, 'metricName' => $metricName, 'resultText' => $output );
        return (%result);
}
